package com.gic.auth.dao.mapper;

import com.gic.auth.dto.UserListDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.UserListQO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TabSysUserMapper {
    /**
     * 根据主键删除
     *
     * @param userId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer userId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysUser record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysUser record);

    /**
     * 根据主键查询
     *
     * @param userId 主键
     * @return 实体对象
     */
    TabSysUser selectByPrimaryKey(Integer userId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysUser record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysUser record);

    /**
     * 手机号码查询个数
     * @param phone
     * @param userId
     * @param enterpriseId
     * @return
     */
    int countByPhone(@Param("phone") String phone, @Param("userId") Integer userId,
            @Param("enterpriseId") Integer enterpriseId);

    /**
     * 手机号码查询个数
     * @Title: countByPhoneAndLoginType 
     * @Description:
      * @author guojuxing
     * @param phone
     * @param userId
     * @param enterpriseId 
     * @param loginType
     * @return  int 
      */
    int countByPhoneAndLoginType(@Param("phone") String phone, @Param("userId") Integer userId,
                     @Param("enterpriseId") Integer enterpriseId, @Param("loginType") Integer loginType);

    TabSysUser selectByEnterpriseId(@Param("enterpriseId") Integer enterpriseId);

    List<TabSysUser> listAllUserByPhoneNumber(@Param("phoneNumber") String phoneNumber);

    TabSysUser login(@Param("phoneNumber") String phoneNumber, @Param("password") String password,
            @Param("enterpriseId") Integer enterpriseId);

    /**
     * 列表查询
     * @Title: listUser 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<UserListDTO> listUser(UserListQO params);

    /**
     * 逻辑删除
     * @Title: deleteById 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  int 
      */
    int deleteById(@Param("userId") Integer userId);

    Integer getFirstNotInUserId(@Param("enterpriseId") Integer enterpriseId, @Param("ids") List<Integer> userIdList);

    Integer countUserByUserIds(@Param("ids") List<Integer> userIdList);

    List<TabSysUser> listUserByIdList(@Param("ids") List<Integer> userIdList);

    /**
     * 查询不在范围内的用户
     * @Title: listUserNotInIdList 
     * @Description:
      * @author guojuxing
     * @param userIdList 
     * @param search
     * @param enterpriseId 
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<TabSysUser> listUserNotInIdList(@Param("ids") List<Integer> userIdList, @Param("search") String search,
            @Param("enterpriseId") Integer enterpriseId);

    /**
     * 查询所有的用户，gic管理员/运营实施管理员
     * @Title: listAllUser 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  java.util.List<java.lang.Integer> 
      */
    List<Integer> listAllUser(@Param("enterpriseId") Integer enterpriseId);
}