package com.gic.auth.service.outer.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.gic.auth.constant.AccountGroupMemberTypeEnum;
import com.gic.auth.entity.TabSysAccountGroupRel;
import com.gic.auth.service.AccountGroupRelService;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.service.EnterpriseApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.SignConstants;
import com.gic.auth.dto.AppDTO;
import com.gic.auth.dto.CollaboratorDTO;
import com.gic.auth.dto.CollaboratorListDTO;
import com.gic.auth.entity.TabSysCollaborator;
import com.gic.auth.entity.TabSysCollaboratorRoleRel;
import com.gic.auth.qo.CollaboratorListQO;
import com.gic.auth.service.CollaboratorApiService;
import com.gic.auth.service.CollaboratorRoleRelService;
import com.gic.auth.service.CollaboratorService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.dto.ApplicationSubDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.ApplicationSubApiService;

@Service("collaboratorApiService")
public class CollaboratorApiServiceImpl implements CollaboratorApiService {
    @Autowired
    private CollaboratorService collaboratorService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private ApplicationSubApiService applicationSubApiService;
    @Autowired
    private CollaboratorRoleRelService collaboratorRoleRelService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private AccountGroupRelService accountGroupRelService;

    @Override
    public ServiceResponse<Integer> save(CollaboratorDTO dto) {
        //新增参数校验
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, CollaboratorDTO.SaveValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }

        collaboratorService.save(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<CollaboratorListDTO>> page(CollaboratorListQO params) {
        if (params.getAccountGroupId() != null) {
            //查询分组下的管理员列表
            List<TabSysAccountGroupRel> list = accountGroupRelService.listByAccountGroupId(params.getAccountGroupId(),
                    AccountGroupMemberTypeEnum.COLLABORATOR.getCode());
            if (CollectionUtils.isNotEmpty(list)) {
                List<Integer> userIdList = new ArrayList<>(list.size());
                for (TabSysAccountGroupRel temp : list) {
                    userIdList.add(temp.getUserId());
                }
                params.setCollaboratorIdList(userIdList);
            }
            //如果是空的，则返回空
            if (CollectionUtils.isEmpty(params.getCollaboratorIdList())) {
                return ServiceResponse.success(new Page<>());
            }

        }
        com.github.pagehelper.Page<CollaboratorListDTO> page = collaboratorService.page(params);
        Page<CollaboratorListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<AppDTO>> listApp(Integer enterpriseId) {
        ServiceResponse<List<ApplicationDTO>> appResult = appTokenApiService
                .listApplicationByEnterpriseId(Long.valueOf(enterpriseId), null);
        if (appResult.isSuccess()) {
            List<ApplicationDTO> appList = appResult.getResult();
            if (CollectionUtils.isNotEmpty(appList)) {
                List<AppDTO> result = new ArrayList<>(appList.size());
                for (ApplicationDTO app : appList) {
                    AppDTO temp = new AppDTO();
                    temp.setAppId(app.getApplicationId().toString());
                    temp.setAppName(app.getName());
                    result.add(temp);
                }
                return ServiceResponse.success(result);
            }
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<List<AppDTO>> listSubApp(String appId, Integer enterpriseId) {
        ServiceResponse<List<ApplicationSubDTO>> subAppResult = applicationSubApiService.listSubApp(enterpriseId,
                Long.valueOf(appId));
        if (subAppResult.isSuccess()) {
            List<ApplicationSubDTO> subAppList = subAppResult.getResult();
            if (CollectionUtils.isNotEmpty(subAppList)) {
                List<AppDTO> result = new ArrayList<>(subAppList.size());
                for (ApplicationSubDTO subApp : subAppList) {
                    AppDTO temp = new AppDTO();
                    temp.setAppId(subApp.getApplicationId().toString());
                    temp.setAppName(subApp.getSubAppName());
                    result.add(temp);
                }
                return ServiceResponse.success(result);
            }
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<CollaboratorDTO> getDetail(Integer collaboratorId) {
        TabSysCollaborator record = collaboratorService.getById(collaboratorId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "协作人ID有误，查无数据");
        }
        CollaboratorDTO result = EntityUtil.changeEntityNew(CollaboratorDTO.class, record);
        List<TabSysCollaboratorRoleRel> menuList = collaboratorRoleRelService.listByCollaboratorId(collaboratorId);
        if (CollectionUtils.isNotEmpty(menuList)) {
            List<Integer> menuIdList = new ArrayList<>(menuList.size());
            for (TabSysCollaboratorRoleRel menu : menuList) {
                menuIdList.add(menu.getMenuId());
            }
            result.setMenuIdList(menuIdList);
        }
        if (StringUtils.isNotBlank(result.getSubAppIds())) {
            result.setSubAppIdList(strToList(result.getSubAppIds()));
        }
        if (StringUtils.isNotBlank(result.getResourceIds())) {
            result.setResourceIdList(strToListOfInteger(result.getResourceIds()));
        }
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService
                .getEnterpriseById(record.getCollaborationEnterpriseId());
        if (enterpriseResult.isSuccess()) {
            result.setCollaborationEnterpriseName(enterpriseResult.getResult().getEnterpriseName());
        }
        return ServiceResponse.success(result);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<String> auth(String subAppIds, String menuIds, String resourcesIds, Integer collaboratorId) {
        TabSysCollaborator record = collaboratorService.getById(collaboratorId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "协作人ID有误，查无数据");
        }
        CollaboratorDTO dto = new CollaboratorDTO();
        //更新资源权限
        if (StringUtils.isNotBlank(subAppIds)) {
            dto.setSubAppIds(commaToUnderline(subAppIds));
        }
        if (StringUtils.isNotBlank(resourcesIds)) {
            dto.setResourceIds(commaToUnderline(resourcesIds));
        }
        dto.setCollaboratorId(collaboratorId);
        collaboratorService.update(dto);
        //更新操作权限
        if (StringUtils.isNotBlank(menuIds)) {
            String[] menuIdArr = menuIds.split(SignConstants.COMMA);
            List<TabSysCollaboratorRoleRel> list = new ArrayList<>(menuIdArr.length);
            Date now = new Date();
            for (String str : menuIdArr) {
                TabSysCollaboratorRoleRel temp = new TabSysCollaboratorRoleRel();
                temp.setCollaboratorId(collaboratorId);
                temp.setCreateTime(now);
                temp.setUpdateTime(now);
                temp.setMenuId(Integer.parseInt(str));
                temp.setStatus(1);
                list.add(temp);
            }
            collaboratorRoleRelService.deleteByCollaboratorId(collaboratorId);
            collaboratorRoleRelService.saveFetch(list);
        }
        return ServiceResponse.success(record.getCollaboratorName());
    }

    @Override
    public ServiceResponse<Void> cancel(Integer collaboratorId) {
        TabSysCollaborator record = collaboratorService.getById(collaboratorId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "协作人ID有误，查无数据");
        }
        CollaboratorDTO dto = new CollaboratorDTO();
        dto.setCollaboratorId(collaboratorId);
        dto.setStatus(0);
        collaboratorService.update(dto);
        return ServiceResponse.success();
    }

    /**
     * 英文逗号隔开的字符串转为下划线隔开
     * @param string
     * @return
     */
    private static String commaToUnderline(String string) {
        String[] strArr = string.split(SignConstants.COMMA);
        StringBuilder sb = new StringBuilder(SignConstants.UNDERLINE);
        for (String str : strArr) {
            sb.append(str).append(SignConstants.UNDERLINE);
        }
        return sb.toString();
    }

    private static List<String> strToList(String string) {
        String[] strArr = string.split(SignConstants.UNDERLINE);
        List<String> list = new ArrayList<>(strArr.length);
        for (String str : strArr) {
            list.add(str);
        }
        return list;
    }
    private static List<Integer> strToListOfInteger(String string) {
        String[] strArr = string.split(SignConstants.UNDERLINE);
        List<Integer> list = new ArrayList<>(strArr.length);
        for (String str : strArr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            list.add(Integer.parseInt(str));
        }
        return list;
    }
}
