package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.OrderOrAppEnum;
import com.gic.auth.dto.OrderAppResourceDTO;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.entity.TabSysOrderAppResource;
import com.gic.auth.entity.TabSysResource;
import com.gic.auth.service.OrderAppResourceService;
import com.gic.auth.service.ResourceApiService;
import com.gic.auth.service.ResourceService;
import com.gic.auth.service.UserResourceService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/28 18:28
 */
@Service("resourceApiService")
public class ResourceApiServiceImpl implements ResourceApiService {

    private Logger logger = LogManager.getLogger(ResourceApiServiceImpl.class);

    @Autowired
    private ResourceService resourceService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private OrderAppResourceService orderAppResourceService;

    @Override
    @Transactional
    public ServiceResponse<Integer> saveOrUpdateResource(ResourceDTO resourceDTO) {
        boolean repeat = resourceService.isRepeatByResourceName(resourceDTO.getEnterpriseId(), resourceDTO.getResourceName(), resourceDTO.getResourceId());
        if (repeat) {
            logger.info("enterpriseId:{} , resourceName:{}, 资源组名称重复 ", resourceDTO.getEnterpriseId(), resourceDTO.getResourceName());
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称重复");
        }
        OrderAppResourceDTO orderDTO = new OrderAppResourceDTO();
        orderDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        orderDTO.setType(OrderOrAppEnum.ORDER.getCode());
        orderDTO.setContent(resourceDTO.getOrderResourceJson());
        Integer order = orderAppResourceService.save(orderDTO);
        resourceDTO.setOrderResource(order);

        OrderAppResourceDTO appDTO = new OrderAppResourceDTO();
        appDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        appDTO.setType(OrderOrAppEnum.ORDER.getCode());
        appDTO.setContent(resourceDTO.getAppResourceJson());
        Integer app = orderAppResourceService.save(appDTO);
        resourceDTO.setAppResource(app);

        OrderAppResourceDTO smsSignDTO = new OrderAppResourceDTO();
        smsSignDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        smsSignDTO.setType(OrderOrAppEnum.SMS_SIGN.getCode());
        smsSignDTO.setContent(resourceDTO.getSmsSignResourceJson());
        Integer smsSign = orderAppResourceService.save(smsSignDTO);
        resourceDTO.setSmsSignResource(smsSign);

        if (resourceDTO.getResourceId() == null) {
            // save
            resourceService.save(resourceDTO);
        } else {
            // update
            // 删除老的订单资源和应用资源
            TabSysResource resource = resourceService.getResource(resourceDTO.getResourceId());
            orderAppResourceService.del(resource.getOrderResource());
            orderAppResourceService.del(resource.getAppResource());
            resourceService.update(resourceDTO);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> delResource(Integer resourceId) {
        TabSysResource resource = resourceService.getResource(resourceId);
        boolean exist = this.userResourceService.existUserResource(resource.getEnterpriseId(), resource.getResourceId());
        if (exist) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该资源组正在使用中，不能删除");
        }
        Integer line = resourceService.delResource(resourceId);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<ResourceDTO>> listResource(ResourceDTO resourceDTO, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabSysResource> page = resourceService.listResource(resourceDTO, pageNum, pageSize);
        Page<ResourceDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, ResourceDTO.class);
        List<ResourceDTO> resourceList = resultPage.getResult();
        if (CollectionUtils.isNotEmpty(resourceList)) {
            List<Integer> resourceIds = resourceList.stream().map(ResourceDTO::getResourceId).collect(Collectors.toList());
            Map<Integer, Integer> map = userResourceService.countByResourceIds(resourceDTO.getEnterpriseId(), resourceIds);
            for (ResourceDTO resource : resourceList) {
                Integer count = map.get(resource.getResourceId());
                resource.setUserResourceCount(count == null ? 0 : count);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<ResourceDTO> getResource(Integer resourceId) {
        TabSysResource resource = resourceService.getResource(resourceId);
        if(resource == null){
            return ServiceResponse.success();
        }
        ResourceDTO resourceDTO = EntityUtil.changeEntityByOrika(ResourceDTO.class, resource);
        if (resourceDTO.getOrderResource() != null) {
            TabSysOrderAppResource orderResource = orderAppResourceService.getById(resourceDTO.getOrderResource());
            resourceDTO.setOrderResourceJson(orderResource.getContent());
        }
        if (resourceDTO.getAppResource() != null) {
            TabSysOrderAppResource appResource = orderAppResourceService.getById(resourceDTO.getAppResource());
            resourceDTO.setAppResourceJson(appResource.getContent());
        }
        if (resourceDTO.getSmsSignResource() != null) {
            TabSysOrderAppResource smsSignResource = orderAppResourceService.getById(resourceDTO.getSmsSignResource());
            resourceDTO.setSmsSignResourceJson(smsSignResource.getContent());
        }
        return ServiceResponse.success(resourceDTO);
    }
}
