package com.gic.auth.web.controller;

import java.util.*;

import com.gic.api.base.commons.Page;
import com.gic.auth.web.vo.LogListVO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuTypeEnum;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuItemApiService;
import com.gic.auth.web.vo.UserOperationItemAuthVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
public class MenuController {

    private final static Logger LOGGER = LogManager.getLogger(MenuController.class);
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private MenuItemApiService menuItemApiService;

    @RequestMapping("login-user-menu")
    public RestResponse loginUserMenu(Integer currentMenu) {
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        ServiceResponse<List<MenuDTO>> userMenu = this.menuApiService.getUserMenuOfGic(userId, currentMenu,
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getVersionCode());
        if (userMenu.isSuccess()) {
            return RestResponse.success(userMenu.getResult());
        }
        return EnterpriseRestResponse.failure(userMenu);
    }

    @RequestMapping("login-user-operation-item")
    public RestResponse loginUserOperationItem() {
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        return ResultControllerUtils.commonResult(
                this.menuApiService.getNoAuthOperationItemOfGic(userId,
                        UserDetailUtils.getUserDetail().getEnterpriseInfo().getVersionCode()),
                UserOperationItemAuthVO.class);
    }

    /**
     * 获取的页面树
     * @Title: listMenuPageTree 
     * @Description:
      * @author guojuxing
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-log-menu-tree")
    public RestResponse listMenuPageTree() {
        MenuListQO params = new MenuListQO();
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        ServiceResponse<List<MenuDTO>> result = menuApiService.listMenuTree(params);
        if (result.isSuccess()) {
            return RestResponse
                    .success(changePageListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 分页查询操作日志
     * @Title: listOperationLog 
     * @Description:
      * @author guojuxing
     * @param startTime
     * @param endTime
     * @param menuId
     * @param search 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-log")
    public RestResponse listOperationLog(String startTime, String endTime, Integer menuId, String search) {
        Page<LogListVO> result = new Page<>();
        LogListVO vo = new LogListVO();
        vo.setContent("北丐测试");
        vo.setLogId(1234567899L);
        vo.setLogTime(new Date());
        vo.setOperationObject("北丐");
        vo.setProject("北丐-北丐测试");
        vo.setUserName("北丐");

        List<LogListVO> list = new ArrayList<>(1);
        list.add(vo);
        result.setResult(list);
        return RestResponse.success(result);
    }

    public static List<Map<String, Object>> changePageListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                if (storeGroupId == menuDTO.getParentId() && menuDTO.getLevel().intValue() <= 2) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("menuCode", menuDTO.getMenuCode());
                    data.put("parentCode", menuDTO.getParentCode());
                    data.put("parentId", menuDTO.getParentId());
                    List<Map<String, Object>> children = changePageListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

}
